@@ -141,12 +141,15 @@ fn main() {
     }}\n\
     \npackage() {{\n    \
         cd \"tmobile-internet-tools-$pkgver\"\n    \
-        mkdir -p \"$pkgdir/usr/share/tmobile-internet-tools\"\n    \
+        mkdir -p \"$pkgdir/usr/share/tmobile-internet-tools/\"\n    \
+        cp -r webui/ \"$pkgdir/usr/share/tmobile-internet-tools/\"\n    \
+        chmod -R 755 \"$pkgdir/usr/share/tmobile-internet-tools/webui/\"\n    \
         mkdir -p \"$pkgdir/usr/share/man/man1/\"\n    \
         mkdir -p \"$pkgdir/usr/lib/systemd/system\"\n    \
         install -Dt \"$pkgdir/usr/bin\" -m755 target/release/gatewaymon\n    \
         install -Dt \"$pkgdir/usr/bin\" -m755 target/release/tmocli\n    \
         install -Dt \"$pkgdir/usr/bin\" -m755 target/release/tmotop\n    \
+        install -Dt \"$pkgdir/usr/bin\" -m755 target/release/tmo-webui\n    \
         install -Dt \"$pkgdir/usr/share/man/man1\" man/gatewaymon.1\n    \
         install -Dt \"$pkgdir/usr/share/man/man1\" man/tmocli.1\n    \
         install -Dt \"$pkgdir/usr/share/man/man1\" man/tmotop.1\n    \
@@ -154,6 +157,8 @@ fn main() {
         install -Dt \"$pkgdir/usr/lib/systemd/system\" gatewaymon.service\n    \
         install -Dt \"$pkgdir/usr/lib/systemd/user\" gatewaymon-debug.service\n    \
         install -Dt \"$pkgdir/usr/lib/systemd/system\" gatewaymon-debug.service\n    \
+        install -Dt \"$pkgdir/usr/lib/systemd/user\" tmo-webui.service\n    \
+        install -Dt \"$pkgdir/usr/lib/systemd/system\" tmo-webui.service\n    \
         gzip \"$pkgdir/usr/share/man/man1/gatewaymon.1\"\n    \
         gzip \"$pkgdir/usr/share/man/man1/tmotop.1\"\n    \
         gzip \"$pkgdir/usr/share/man/man1/tmocli.1\"\n\
