@@ -1,9 +1,9 @@
 use std::{env, io, process};
 use crate::client::*;
 use crate::{
-    hosts::HostList,
-    config::{Config, GlobalConfig},
-    stats::*
+    devices::DeviceList,
+    stats::*,
+    CLIENT
 };
 use colored::Colorize;
 
@@ -18,27 +18,8 @@ const RATING: [&'static str; 5] = [
 pub fn parse_args() {
     // get the command line arguments of the program
     let args: Vec<String> = env::args().collect();
-    // create the client object and allow the user to enter the admin password if it is incorrect 
-    let mut client = match TrashcanClient::new() {
-        Ok(c) => c,
-        Err(_) => {
-            let mut passwd = String::new();
-            let mut config = GlobalConfig::load();
-            println!("WARNING! Your password will be visible!!");
-            println!("Gateway admin password: ");
-            io::stdin().read_line(&mut passwd).unwrap();
-            passwd.retain(|c| c !='\n');
-            config.password = passwd;
-            match TrashcanClient::new() {
-                Ok(c) => c,
-                Err(_) => {
-                    let msg = "Unable to Log into the gateway. Is your admin password correct?".bright_red();
-                    eprintln!("{}", msg);
-                    return;
-                }
-            }
-        }
-    };
+
+    let mut client = CLIENT.write().unwrap();
     
     if args.len() <= 1 {
         show_stats(&get_stats(&client));
@@ -63,7 +44,7 @@ pub fn parse_args() {
         "-v" => {
             println!("T-Mobile Home Internet Tools\nVersion {}", env!("CARGO_PKG_VERSION"));
         },
-        "devices" => show_devices(client),
+        "devices" => show_devices(&client),
         "device" => {
             let arg = match args.get(2) {
                 Some(a) => a.to_string(),
@@ -72,13 +53,13 @@ pub fn parse_args() {
                     return;
                 }
             };
-            show_device(client, arg);
+            show_device(&client, arg);
         },
         "stats" => show_stats(&get_stats(&client)),
         "data-usage" | "data" => show_data(&get_stats(&client)),
         "networks" | "net" => show_networks(&get_stats(&client)),
         "connection" | "con" => show_con_stat(&get_stats(&client)),
-        "info" => show_info(client),
+        "info" => show_info(&client),
         "reboot" => {
             let arg = match args.get(2) {
                 Some(a) => a,
@@ -112,7 +93,7 @@ pub fn parse_args() {
     }
 }
 
-fn show_info(client: TrashcanClient) {
+fn show_info(client: &TrashcanClient) {
     match client.get_info() {
         Ok(i) => {
             println!("Gateway Info:");
@@ -173,9 +154,9 @@ fn show_con_stat(stats: &GatewayStats) {
     println!("Strength: {} dBm ({})", stats.info_4g.strength, RATING[(stats.info_4g.bars - 1) as usize]);
 }
 
-fn show_device(client: TrashcanClient, arg: String) {
+fn show_device(client: &TrashcanClient, arg: String) {
     let hosts = get_devices(&client);
-    for host in &hosts.hosts {
+    for host in &hosts.devices {
         if host.get_name() == arg || host.get_ip() == arg || host.get_mac_addr() == arg{
             println!("{}", host.get_name().bright_magenta());
             println!("{}", host.get_ip());
@@ -186,9 +167,9 @@ fn show_device(client: TrashcanClient, arg: String) {
     }
 }
 
-fn show_devices(client: TrashcanClient) {
+fn show_devices(client: &TrashcanClient) {
     let hosts = get_devices(&client);
-    for host in &hosts.hosts {
+    for host in &hosts.devices {
         println!("{}", host.get_name().bright_magenta());
         println!("{}", host.get_ip());
         if !host.is_gateway() {
@@ -199,8 +180,8 @@ fn show_devices(client: TrashcanClient) {
     print!("\x08");
 }
 
-fn get_devices(client: &TrashcanClient) -> HostList {
-    match client.get_hosts() {
+fn get_devices(client: &TrashcanClient) -> DeviceList {
+    match client.get_devices() {
         Ok(h) => h,
         Err(why) => {
             let msg = format!("Error: Unable to get devices connected to gateway! {}", why).bright_red();
@@ -212,12 +193,12 @@ fn get_devices(client: &TrashcanClient) -> HostList {
 
 // get the gateway's stats
 fn get_stats(client: &TrashcanClient) -> GatewayStats {
-        match client.get_all_stats() {
-            Ok(s) => s,
-            Err(why) => {
-                let msg = format!("Error: Unable to get gateway stats! {}", why).bright_red();
-                eprintln!("{}", msg);
-                process::exit(255);
-            }
+    match client.get_all_stats() {
+        Ok(s) => s,
+        Err(why) => {
+            let msg = format!("Error: Unable to get gateway stats! {}", why).bright_red();
+            eprintln!("{}", msg);
+            process::exit(255);
         }
+    }
 }
\ No newline at end of file
