@@ -18,8 +18,8 @@ use rust_utils::{
 use lazy_static::lazy_static;
 use token::AuthToken;
 use crate::{
-    config::{GlobalConfig, GatewayConfig, HostConfig},
-    hosts::HostList,
+    config::{GlobalConfig, GatewayConfig, DeviceConfig},
+    devices::DeviceList,
     stats::*,
     LOG,
     DEBUG,
@@ -37,10 +37,25 @@ lazy_static! {
     static ref LOG_DIR: String = format!("{}/.local/share/tmo-tools", env::var("HOME").expect("Where the hell is your home folder?!"));
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum ClientError {
     LoginFailed,
-    GatewayUnreachable(String)
+    GatewayUnreachable(String),
+    JSONParseError(String, String)
+}
+
+pub type ClientResult<T> = Result<T, ClientError>;
+
+impl actix_web::ResponseError for ClientError { }
+
+impl fmt::Display for ClientError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match *self {
+            Self::LoginFailed => write!(f, "Unable to log into the T-Mobile gateway. Is the username/password in global.toml correct? Or is there a web client logged in elsewhere?"),
+            Self::GatewayUnreachable(ref why) => write!(f, "Unable to reach the T-Mobile gateway: {}", why),
+            Self::JSONParseError(ref why, ref data) => write!(f, "JSON data failed to parse correctly!\nError: {}\nJSON data:\n{}", why, data)
+        }
+    }
 }
 
 #[derive(Copy, Clone)]
@@ -50,40 +65,31 @@ enum AuthMode {
     None
 }
 
-impl fmt::Display for ClientError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            Self::LoginFailed => write!(f, "Unable to log into the T-Mobile gateway. Is the username/password in global.toml correct? Or is there a web client logged in elsewhere?"),
-            Self::GatewayUnreachable(ref why) => write!(f, "Unable to reach the T-Mobile gateway: {}", why)
-        }
-    }
-}
-
 pub struct TrashcanClient {
     pub config: GlobalConfig,
     pub gw_config: GatewayConfig,
-    pub hst_config: HostConfig,
+    pub dev_config: DeviceConfig,
     client: Client,
-    jar: Arc<Jar>
+    jar: Arc<Jar>,
 }
 
 impl TrashcanClient {
     // create a new client
-    pub fn new() -> Result<TrashcanClient, ClientError> {
+    pub fn new() -> TrashcanClient{
         let config = GlobalConfig::load();
         let jar = Arc::new(Jar::default());
         let client = Client::builder().connect_timeout(Duration::from_secs(10)).timeout(Duration::from_secs(10)).cookie_store(true).cookie_provider(jar.clone()).build().unwrap();
-        Ok(TrashcanClient {
+        TrashcanClient {
             config,
             gw_config: GatewayConfig::load(),
-            hst_config: HostConfig::load(),
+            dev_config: DeviceConfig::load(),
             client: client.clone(),
             jar
-        })
+        }
     }
 
     // request the gateway to reboot
-    pub fn reboot_gateway(&mut self, print_stdout: bool) -> Result<(), ClientError> {
+    pub fn reboot_gateway(&mut self, print_stdout: bool) -> ClientResult<()> {
         LOG.line(LogLevel::Info, "Rebooting gateway...", print_stdout);
         self.reboot_gateway_no_log()
     }
@@ -94,16 +100,16 @@ impl TrashcanClient {
     }
 
     // get a list of devices and their ipv4 addresses connected to the gateway
-    pub fn get_hosts(&self) -> Result<HostList, ClientError> {
+    pub fn get_devices(&self) -> ClientResult<DeviceList> {
         let response = self.get("dashboard_device_info_status_web_app.cgi", AuthMode::None)?;
         let json = parse_json(&response);
-        Ok(HostList::get(json, &self.hst_config)?)
+        Ok(DeviceList::get(json?, &self.dev_config)?)
     }
 
     // get the 5G metrics
-    pub fn get_5g_stats(&self) -> Result<CellInfo, ClientError> {
+    pub fn get_5g_stats(&self) -> ClientResult<CellInfo> {
         let response = self.get("fastmile_radio_status_web_app.cgi", AuthMode::None)?;
-        let json = parse_json(&response);
+        let json = parse_json(&response)?;
         let bars = json["cell_LTE_stats_cfg"][0]["stat"]["RSRPStrengthIndexCurrent"].as_u8().expect("Value not found!");
         let strength = json["cell_LTE_stats_cfg"][0]["stat"]["RSRPCurrent"].as_i32().expect("Value not found!");
         let band = json["cell_5G_stats_cfg"][0]["stat"]["Band"].as_str().expect("Value not found!").to_string();
@@ -116,9 +122,9 @@ impl TrashcanClient {
     }
 
     // get the LTE metrics
-    pub fn get_4g_stats(&self) -> Result<CellInfo, ClientError> {
+    pub fn get_4g_stats(&self) -> ClientResult<CellInfo> {
         let response = self.get("fastmile_radio_status_web_app.cgi", AuthMode::None)?;
-        let json = parse_json(&response);
+        let json = parse_json(&response)?;
         let bars = json["cell_5G_stats_cfg"][0]["stat"]["RSRPStrengthIndexCurrent"].as_u8().expect("Value not found!");
         let strength = json["cell_5G_stats_cfg"][0]["stat"]["RSRPCurrent"].as_i32().expect("Value not found!");
         let band = json["cell_LTE_stats_cfg"][0]["stat"]["Band"].as_str().expect("Value not found!").to_string();
@@ -136,16 +142,20 @@ impl TrashcanClient {
         if response.is_err() {
             return false;
         }
-        let json = parse_json(&response.unwrap());
-        let val = json["connection_status"][0]["ConnectionStatus"].as_usize();
-        if val.is_none() {
-            return false;
-        }
+        let json_res = parse_json(&response.unwrap());
 
-        val.unwrap() == 1
+        if let Ok(json) = json_res {
+            let val = json["connection_status"][0]["ConnectionStatus"].as_usize();
+            if val.is_none() {
+                return false;
+            }
+    
+            return val.unwrap() == 1
+        }
+        false
     }
 
-    pub fn get_all_stats(&self) -> Result<GatewayStats, ClientError> {
+    pub fn get_all_stats(&self) -> ClientResult<GatewayStats> {
         let conn_status = self.get_conn_status();
         let info_5g = self.get_5g_stats()?;
         let info_4g = self.get_4g_stats()?;
@@ -153,7 +163,7 @@ impl TrashcanClient {
         let upload = self.get_upload()?;
         let sig_tracker = tracker::SignalTracker::get();
         let networks = self.get_networks()?;
-        let devices = self.get_hosts()?;
+        let devices = self.get_devices()?;
 
         Ok(GatewayStats {
             conn_status,
@@ -167,28 +177,81 @@ impl TrashcanClient {
         })
     }
 
+    pub fn get_adv_stats(&self) -> ClientResult<AdvancedGatewayStats> {
+        let response = self.get("fastmile_radio_status_web_app.cgi", AuthMode::None)?;
+        let json = parse_json(&response)?;
+
+        // APN info
+        let apn_name = json["apn_cfg"][0]["APN"].as_str().unwrap().to_string();
+        let apn_ip4 = json["apn_cfg"][0]["X_ALU_COM_IPAddressV4"].as_str().unwrap().to_string();
+        let apn_ip6 = json["apn_cfg"][0]["X_ALU_COM_IPAddressV6"].as_str().unwrap().to_string();
+
+        // 5G cell info
+        let band_5g = json["cell_5G_stats_cfg"][0]["stat"]["Band"].as_str().unwrap().to_string();
+        let rsrp_5g = json["cell_LTE_stats_cfg"][0]["stat"]["RSRPCurrent"].as_i32().unwrap();
+        let snr_5g = json["cell_5G_stats_cfg"][0]["stat"]["SNRCurrent"].as_i32().unwrap();
+        let rsrq_5g = json["cell_LTE_stats_cfg"][0]["stat"]["RSRQCurrent"].as_i32().unwrap();
+        let rssi = json["cell_LTE_stats_cfg"][0]["stat"]["RSSICurrent"].as_i32().unwrap();
+        
+        // 4G cell info
+        let band_4g = json["cell_LTE_stats_cfg"][0]["stat"]["Band"].as_str().unwrap().to_string();
+        let rsrp_4g = json["cell_5G_stats_cfg"][0]["stat"]["RSRPCurrent"].as_i32().unwrap();
+        let snr_4g = json["cell_LTE_stats_cfg"][0]["stat"]["SNRCurrent"].as_i32().unwrap();
+        let rsrq_4g = json["cell_5G_stats_cfg"][0]["stat"]["RSRQCurrent"].as_i32().unwrap();
+
+        // SIM info
+        let response = self.get("fastmile_statistics_status_web_app.cgi", AuthMode::Web)?;
+        let sim_json = parse_json(&response)?;
+        let imsi = sim_json["sim_cfg"][0]["IMSI"].as_str().unwrap().parse().unwrap();
+        let iccid = sim_json["sim_cfg"][0]["ICCID"].as_str().unwrap().parse().unwrap();
+        let imei = sim_json["network_cfg"][0]["IMEI"].as_str().unwrap().parse().unwrap();
+
+        Ok(AdvancedGatewayStats {
+            apn_name,
+            apn_ip4,
+            apn_ip6,
+            band_5g,
+            rsrp_5g,
+            snr_5g,
+            rsrq_5g,
+            rssi,
+            band_4g,
+            rsrp_4g,
+            snr_4g,
+            rsrq_4g,
+            imsi,
+            iccid,
+            imei
+        })
+    }
+
     // gets info about the gateway
-    pub fn get_info(&self) -> Result<GatewayInfo, ClientError> {
+    pub fn get_info(&self) -> ClientResult<GatewayInfo> {
         let response = self.get("device_status_web_app.cgi", AuthMode::Web)?;
-        let json = parse_json(&response);
+        let json = parse_json(&response)?;
+        let response = self.get("fastmile_statistics_status_web_app.cgi", AuthMode::Web)?;
+        let sim_json = parse_json(&response)?;
         let vendor = json["Vendor"].as_str().unwrap().to_string();
         let ser_num = json["SerialNumber"].as_str().unwrap().to_string();
         let hw_ver = json["HardwareVersion"].as_str().unwrap().to_string();
         let sw_ver = json["SoftwareVersion"].as_str().unwrap().to_string();
         let uptime = json["UpTime"].as_usize().unwrap();
-        Ok(GatewayInfo {
+        let imei = sim_json["network_cfg"][0]["IMEI"].as_str().unwrap().parse().unwrap();
+        let info = GatewayInfo {
             vendor,
             ser_num,
             hw_ver,
             sw_ver,
-            uptime
-        })
+            uptime,
+            imei
+        };
+        Ok(info)
     }
 
     pub fn reload_config(&mut self) {
         self.config = GlobalConfig::load();
         self.gw_config = GatewayConfig::load();
-        self.hst_config = HostConfig::load();
+        self.dev_config = DeviceConfig::load();
     }
 
     pub fn is_valid_net(&self) -> bool {
@@ -196,7 +259,7 @@ impl TrashcanClient {
     }
 
     // POST request to the gateway
-    fn post(&self, res: &str, mode: AuthMode) -> Result<(), ClientError> {
+    fn post(&self, res: &str, mode: AuthMode) -> ClientResult<()> {
         let addr = format!("http://{}/{}", self.config.gateway_ip, res);
         let req = self.client.post(addr);
         let req_r = match mode {
@@ -218,7 +281,7 @@ impl TrashcanClient {
     }
 
     // GET request to the gateway
-    fn get(&self, res: &str, mode: AuthMode) -> Result<String, ClientError> {
+    fn get(&self, res: &str, mode: AuthMode) -> ClientResult<String> {
         let addr = format!("http://{}/{}", self.config.gateway_ip, res);
         let req = self.client.get(addr);
         let req_r = match mode {
@@ -228,7 +291,6 @@ impl TrashcanClient {
             },*/
             AuthMode::Web => {
                 let _token = self.gen_web_token()?;
-                //.form(&token.get_data())
                 req.send()
             },
 
@@ -241,7 +303,7 @@ impl TrashcanClient {
         }
     }
     
-    fn gen_web_token(&self) -> Result<AuthToken, ClientError> {
+    fn gen_web_token(&self) -> ClientResult<AuthToken> {
         let token_r = AuthToken::get_web(&self.client, &self.config.username, &self.config.password);
         if let Ok(token) = token_r {
             token.put_cookies_in_jar(&self.jar);
@@ -263,25 +325,23 @@ impl TrashcanClient {
         }
     }*/
 
-    fn get_upload(&self) -> Result<u128, ClientError> {
+    fn get_upload(&self) -> ClientResult<u128> {
         let response = self.get("fastmile_radio_status_web_app.cgi", AuthMode::None)?;
-        let json = parse_json(&response);
+        let json = parse_json(&response)?;
         Ok(json["cellular_stats"][0]["BytesSent"].as_u64().expect("Value not found!") as u128)
     }
 
-    fn get_download(&self) -> Result<u128, ClientError> {
+    fn get_download(&self) -> ClientResult<u128> {
         let response = self.get("fastmile_radio_status_web_app.cgi", AuthMode::None)?;
-        let json = parse_json(&response);
+        let json = parse_json(&response)?;
         Ok(json["cellular_stats"][0]["BytesReceived"].as_u64().expect("Value not found!") as u128)
     }
 
-    fn get_networks(&self) -> Result<Vec<NetworkInfo>, ClientError> {
+    fn get_networks(&self) -> ClientResult<Vec<NetworkInfo>> {
         let response = self.get("statistics_status_web_app.cgi", AuthMode::None)?;
-        let json = parse_json(&response);
-
+        let json = parse_json(&response)?;
         let eth_networks = json["LAN"].members();
         let mut networks: Vec<NetworkInfo> = vec![];
-
         for (i, network_raw) in eth_networks.enumerate() {
             let id = network_raw["Name"].as_str().expect("Value not found!").to_string();
             let name = format!("Ethernet {}", i + 1);
@@ -320,18 +380,11 @@ impl TrashcanClient {
     }
 }
 
-// parse text as json and print the text then panic if there is a parsing error
-fn parse_json(source: &str) -> json::JsonValue {
+// parse text as json
+fn parse_json(source: &str) -> ClientResult<json::JsonValue> {
     match json::parse(source) {
-        Ok(j) => j,
-        Err(why) => {
-            LOG.line(LogLevel::Error, format!("JSON Error: {}", why), false);
-            LOG.line(LogLevel::Error, "Received input:", false);
-            for line in source.lines() {
-                LOG.line(LogLevel::Error, line, false);
-            }
-            panic!("JSON failed to parse correctly!");
-        }
+        Ok(j) => Ok(j),
+        Err(why) => return Err(ClientError::JSONParseError(why.to_string(), source.to_string()))
     }
 }
 
@@ -353,12 +406,14 @@ pub fn get_log_dates() -> Vec<(u32, u32, u32)> {
     }
     let mut uniques = HashSet::new();
     dates.retain(|d| uniques.insert(*d));
+    dates.sort();
+    dates.reverse();
     dates
 }
 
 // get a log file by date
 pub fn get_log(date: (u32, u32, u32)) -> StyledString {
-    let raw = fs::read_to_string(format!("{}/gatewaymon-{}-{}-{}.log", *LOG_DIR, date.0, date.1, date.2)).unwrap_or(" ".to_string());
+    let raw = get_log_plain(date);
     let mut styled = StyledString::new();
     for line in raw.lines() {
         let style = if line.contains("DEBUG]") {
@@ -370,6 +425,9 @@ pub fn get_log(date: (u32, u32, u32)) -> StyledString {
         else if line.contains("ERROR]") {
             ColorStyle::from(Color::Light(BaseColor::Red))
         }
+        else if line.contains("FATAL]") {
+            ColorStyle::from((Color::Light(BaseColor::Red), Color::Dark(BaseColor::Black)))
+        }
         else {
             ColorStyle::from(Color::Dark(BaseColor::Green))  
         };
@@ -379,6 +437,10 @@ pub fn get_log(date: (u32, u32, u32)) -> StyledString {
     styled
 }
 
+pub fn get_log_plain(date: (u32, u32, u32)) -> String {
+    fs::read_to_string(format!("{}/gatewaymon-{}-{}-{}.log", *LOG_DIR, date.0, date.1, date.2)).unwrap_or(" ".to_string())
+}
+
 // check the network environment
 // 0: network is fine
 // 1: gateway needs rebooted
@@ -459,7 +521,7 @@ pub fn nmcli<A: IntoIterator<Item = S>, S: AsRef<OsStr>>(args: A, print_stdout:
 // false: there a was problem
 fn ping_req() -> bool {
     let mut ping_cmd = Command::new("ping");
-    ping_cmd.args(["paradisemod.net", "-c1", "-W10"]);
+    ping_cmd.args(["paradisemod.net", "-c1", "-w10"]);
     let out = ping_cmd.output().unwrap();
 
     if *DEBUG {
