@@ -70,15 +70,19 @@ impl Config for GatewayConfig {
 }
 
 #[derive(Deserialize, Serialize, Clone)]
-pub struct HostConfig {
+pub struct DeviceConfig {
     pub tld: String,
     pub gateway_name: String,
     pub aliases: HashMap<String, String>
 }
 
-impl Default for HostConfig {
-    fn default() -> HostConfig {
-        HostConfig {
+impl DeviceConfig {
+
+}
+
+impl Default for DeviceConfig {
+    fn default() -> DeviceConfig {
+        DeviceConfig {
             // top level domain
             tld: ".local".to_owned(),
 
@@ -91,7 +95,7 @@ impl Default for HostConfig {
     }
 }
 
-impl Config for HostConfig {
-    const FILE_NAME: &'static str = "hosts.toml";
+impl Config for DeviceConfig {
+    const FILE_NAME: &'static str = "devices.toml";
     fn get_save_dir() -> String { format!("{}/.config/tmo-tools", env::var("HOME").expect("Where the hell is your home folder?!")) }
 }
\ No newline at end of file
