@@ -3,31 +3,15 @@ use std::{
     time::{Instant, Duration},
 };
 use chrono::{Timelike, Datelike, Local};
-use rust_utils::{
-    config::Config,
-    logging::LogLevel
-};
-use crate::client::{self, tracker::*,};
-use crate::config::GlobalConfig;
-use crate::{LOG, DEBUG};
+use rust_utils::logging::LogLevel;
+use crate::client::{self, tracker::*};
+use crate::{LOG, DEBUG, CLIENT};
 
 // gateway monitoring daemon (background process)
 // this will check the network environment once every minute and will tell the gateway to reboot if there are any problems
 pub fn init() {
     LOG.line_basic("Starting T-Mobile Gateway Monitor...", true);
-    let mut client = match client::TrashcanClient::new() {
-        Ok(c) => c,
-        Err(error) => {
-            if !client::is_valid_net(&GlobalConfig::load().valid_networks) {
-                LOG.line(LogLevel::Error, "This network is not a T-Mobile home Internet network", true);
-            }
-            else {
-                LOG.line(LogLevel::Error, format!("Error starting T-Mobile Gateway Monitor: {}", error), true);
-            }
-            return;
-        }
-    };
-
+    let mut client = CLIENT.write().unwrap();
     let mut tracker = SignalTracker::get();
 
     LOG.line_basic("Startup complete.", true);
